
#ifndef ALL_H
#define ALL_H

extern "C" {
#include "MonochromeLib.h"
#include "SH4_compatibility.h"
#include "usefull.h"
#include "font.h"
#include <stdlib.h>
}


const int E_STORE = 0;
const int E_SIGN_ACID = 1;
const int E_SIGN_FALLING = 2;
const int E_BORING_GUY = 3;
const int E_STALACTITE = 4;
const int E_LASER = 5;
const int E_MIRROR = 6;
const int E_WALL = 7;
const int E_VINE = 8;
const int E_TREE = 9;
const int E_WIN = 10;
const int E_PRESS = 11;
const int E_ENEMY = 12;
const int E_BOMB = 13;

const int S_GAME = 0;
const int S_STORE = 1;
const int S_DIALOG = 2;
const int S_DEATH = 3;
const int S_WIN = 4;

const int D_CIRCLE = 0;
const int D_STALACTITE = 1;
const int D_BOREDOM = 2;
const int D_DROWNING = 3;
const int D_LASER = 4;
const int D_EXPLOSION = 5;
const int D_POISON = 6;
const int D_FIRE = 7;
const int D_FALLING = 8;
const int D_PRESS = 9;
const int D_WIN = 10;
const int D_RESET = 11;

const int A_DEATH = 64;
const int A_DEATH_MARGIN = 16;

const int AL_LEFT = 0;
const int AL_RIGHT = 1;
const int AL_MIDDLE = 2;

const int U_LINGOT = 0;
const int U_BOOTS = 1;
const int U_WINGS = 2;
const int U_BOTTLES = 3;
const int U_RAY = 4;
const int U_BOMBS = 5;
const int U_TORCH = 6;

const int P_WALL = 0;
const int P_STALACTITE = 1;
const int P_BURNING = 2;
const int P_DEATH = 3;

const int MAX_PARTICLE = 30;
const int MAX_ELEMENT = 50;

class FX {


	public :
		void draw(int cameraX,int cameraY);
		void frame();
		void shake();
		void reset();
		void addParticle(int x,int y, float speedTX, float speedTY, int type);
		FX();
	private :

		int frameShake;
		float* positionX;
		float* positionY;
		float* speedX;
		float* speedY;
		int* particleId;

};

class Element {

	public :
		void start(int xt,int yt, int idt);
		void frame(int quick);
		void draw(int cameraX,int cameraY);
		int shouldDelete();
		void bombExplodingAt(int bombX, int bombY);
		int isAvailable();

	private :
		int x;
		float y;
		int id;
		int burning;
		int falling;
		int frozen;
		int frameCount;
		int broken;
		int activated;
};



class Dialog {

	public :

		Dialog();
		void frame();
		void draw();
		char* getText();
		void start(int w);
		int melting();
		void reset();

	private :
		int page;
		int framePage;
		int which;
		int keyPressed;
};


class Heros {
	public :
		void frame();
		void input();
		void draw(int cameraX,int cameraY);
		int getX();
		int getY();
		float getSpeedY();
		void setX(int newX);
		void setY(int newY);
		int hasUpgrade(int which);
		void addUpgrade(int which);
		Heros();
		void reset();
		int pointInRay(int x, int y);

	private :

		int upgrades[7];
		int oldX[45];
		int oldY[45];
		int pointerOld;
		int x;
		float y;
		float speedY;
		int goingLeft;
		int isGrounded;
		int frameUnderWater;

		int jumpAnim;
		int landingAnim;
		int firingFreezeRay;
		int canDoubleJump;
		int shiftPressed;
		int bombCD;
};


class Store {
	public :
		void frame();
		void draw();
		void open();
		void refreshRemaining();

	private :

		void printDescription(char* str, int startX, int y, int breakX);
		int cursor;
		int money;
		int keyDown;
};


class Map {
  public:
		Map();
		void frame();
    void draw(int cameraX,int cameraY);
		int getTile(int tileX,int tileY);
		int getTilePoint(int tileX,int tileY);
		void setTilePoint(int tileX,int tileY,int id);
		void reset();
		int isSolid(int pointX, int pointY);
		int isWater(int pointX, int pointY);

	private:
		char tiles[4756];
		int sizeX;
		int sizeY;
		int frameCount;

};


class Death {
	public :
		void frame();
		void draw();
		void init(int w,int o);
		int getDeath();
		void reset();
	private :
		int which;
		int frameCount;
		int original;
};


class Game {
  public:
		Game();
    void frame();
    void draw();
		void startDialog(int which);
		void finishDialog(int which);
		int getTotalMoney();
		void die(int which);
		void win();
		void endDying();
		int isDead();
		int getDeath();

	private:
		int test;
		int cameraX;
		int state;
		int cameraY;
		int deathDone[20];
		Map* map;
		Heros* heros;
		Element* elements;
		Dialog* dialog;
		Store* store;
		Death* death;
		int frameSinceReset;
		int frameSinceStart;
		int invertColor;
};



class Single
{
	public :
		static Game* getGame();
		static Heros* getHeros();
		static Element* getElements();
		static Map* getMap();
		static Dialog* getDialog();
		static FX* getFX();

	private:
		static Game* game;
		static Heros* heros;
		static Element elements[MAX_ELEMENT];
		static Map* map;
		static Dialog* dialog;
		static FX* fx;

};

#endif
